from celery import shared_task
from .models import Message, ShiftRegistration
from django.conf import settings
from django.db import transaction
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry
from .sipgate.sms import send as send_sms

global_preferences = global_preferences_registry.manager()


def send(msg):
    if not (
        settings.SIPGATE_SMS_EXTENSION
        and settings.SIPGATE_TOKEN
        and settings.SIPGATE_TOKEN_ID
    ):
        print(f"would send message to {msg.to.phone}\n---\n{msg.text}")
        return

    send_sms(str(msg.to.phone), msg.text)


# cron task to send normal messages(reminders,changes) in batches
@shared_task
def send_messages():
    if not global_preferences["helper__send_sms"]:
        print("sms disabled, not sending")
        return
    with transaction.atomic():
        for msg in Message.objects.select_for_update().filter(sent_at__isnull=True):
            if msg.sent_at:
                continue
            send(msg)
            msg.sent_at = timezone.now()
            msg.save()


# singlemessage so registration links arrive faster.
# will silently fail when messages are disabled.
# those messages will be picked up by the cron send later
# once we have decided to continue sending messages
@shared_task
def send_message(msgid):
    if not global_preferences["helper__send_sms"]:
        return
    with transaction.atomic():
        msg = Message.objects.select_for_update().get(pk=msgid)
        if msg.sent_at:
            return
        send(msg)
        msg.sent_at = timezone.now()
        msg.save()


@shared_task
def send_reminders():
    with transaction.atomic():
        for reg in ShiftRegistration.objects.select_for_update().filter(
            reminder_sent=False,
            shift__start_at__lte=timezone.now()
            + global_preferences["helper__reminder_time"],
        ):
            reg.send_reminder()
