from datetime import timezone
from django.conf import settings
from django.db import transaction
from icalendar import Calendar
from .models import Event, Room, Shift
import requests


def import_calendar(calendar):
    try:
        r = requests.get(calendar.url)
        r.raise_for_status()
    except:
        if settings.DEBUG:
            raise
        return False

    if not r.headers["content-type"].startswith("text/calendar"):
        return False

    try:
        cal = Calendar.from_ical(r.text)

        rooms = {}
        events = {}
        for event in cal.walk("vevent"):
            uid = event.decoded("uid").decode()
            room = (
                event.decoded("location", None) or event.decoded("summary")
            ).decode()
            start = event.decoded("dtstart").astimezone(timezone.utc)
            end = event.decoded("dtend").astimezone(timezone.utc)

            if not uid or not room:
                return False

            rooms[room] = None
            events[uid] = (
                room,
                {
                    "start_at": start,
                    "duration": end - start,
                    "uuid": uid,
                    "calendar": calendar,
                },
            )

        with transaction.atomic():
            for r in Room.objects.filter(name__in=rooms):
                rooms[r.name] = r
            for room, r in rooms.items():
                if r == None:
                    rooms[room] = Room(
                        name=room, required_helpers=0
                    )  # required_helpers=0 ensures a shift in a new room is not displayed until the correct number of required helpers is set
                    rooms[room].save()

            for e in Event.objects.filter(calendar=calendar, uuid__in=events):
                uuid = str(e.uuid)
                room, event = events[uuid]

                e.room = rooms[room]
                e.start_at = event["start_at"]
                e.duration = event["duration"]
                e.save()

                events[uuid] = (room, e)
            for event in events:
                room, e = events[event]
                if not isinstance(e, Event):
                    Event(room=rooms[room], **e).save()

            for event in Event.objects.filter(calendar=calendar).exclude(
                uuid__in=events
            ):
                event.deleted = True
                event.save()
    except:
        if settings.DEBUG:
            raise
        return False

    return True
